import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.io.*;

/**
 * This class represents a Panel that contains the buttons that can pause, load and read
 * and displays the level, score and next peice.
 */
public class SLPanel extends JPanel
{
	// Instance variables
	private NPPanel nextPiecePanel;
	private JPanel scorePanel;
	private JPanel levelPanel;
	private JPanel buttonPanel;
	private JLabel nextPieceLabel;
	private JLabel scoreName;
	private JLabel scoreLabel;
	private JLabel levelName;
	private JLabel levelLabel;
	private JButton saveButton;
	private JButton loadButton;
	private JButton pauseButton;
	private PlayingPanel play;	
	
	/**
	 * Constructor. Creates an new instance of SLPanel.
	 * @param score The game score
	 * @param level The level
	 * @param nextPiece The next piece
	 */
	public SLPanel(int score, int level, Piece nextPiece, PlayingPanel play)
	{
		// Save reference
		this.play = play;
	
		// Set preferred size
		setPreferredSize(new Dimension(150, 100));
		
		// Set layout manager
		setLayout(new BorderLayout());
		
		// Set background color
		setBackground(Color.WHITE);
		repaint();
		
		// Create nextPiecePanel
		nextPiecePanel = new NPPanel(nextPiece);
		nextPieceLabel = new JLabel("Next Piece");
		nextPiecePanel.setPreferredSize(new Dimension(75, 75));
		nextPiecePanel.add(nextPieceLabel);
		nextPiecePanel.setBackground(Color.WHITE);
		
		// Create scorePanel		
		scorePanel = new JPanel();
		scoreName = new JLabel("Score");
		scoreLabel = new JLabel(""+score);
		scorePanel.setPreferredSize(new Dimension(75, 75));
		scorePanel.add(scoreName);
		scorePanel.add(scoreLabel);
		scorePanel.setBackground(Color.WHITE);
		
		// Create levelPanel
		levelPanel = new JPanel();
		levelName = new JLabel("Level");
		levelLabel = new JLabel(""+level);
		levelPanel.setPreferredSize(new Dimension(75, 75));
		levelPanel.add(levelName);
		levelPanel.add(levelLabel);
		levelPanel.setBackground(Color.WHITE);
		
		// Create buttonPanel
		buttonPanel = new JPanel();
		buttonPanel.setLayout(new BorderLayout());
		saveButton = new JButton("Save");
		saveButton.addActionListener(new SaveButtonListener());
		saveButton.setFocusable(false);
		loadButton = new JButton("Load");
		loadButton.addActionListener(new LoadButtonListener());
		loadButton.setFocusable(false);
		pauseButton = new JButton("Pause");
		pauseButton.addActionListener(new PauseButtonListener());
		pauseButton.setFocusable(false);
		buttonPanel.add(saveButton, BorderLayout.WEST);
		buttonPanel.add(loadButton, BorderLayout.CENTER);
		buttonPanel.add(pauseButton, BorderLayout.EAST);
		buttonPanel.setBackground(Color.WHITE);	
		
		// Add to panel
		add(nextPiecePanel, BorderLayout.EAST);
		add(scorePanel, BorderLayout.CENTER);
		add(levelPanel, BorderLayout.WEST);	
		add(buttonPanel, BorderLayout.SOUTH);	
	}
	
	/**
	 * This method updates the level, score and piece. Called from PlayingPanel
	 */
	public void updateSL(int score, int level, Piece nextPiece)
	{
		levelLabel.setText(""+level);
		scoreLabel.setText(""+score);
		nextPiecePanel.updatePiece(nextPiece);
	}
	
	/**
	 * Private class for firing events when the save button is pressed
	 */
	private class SaveButtonListener implements ActionListener
	{
		/**
		 * Method invoked when the button is pressed. Saves the game in a file
		 * specified by the user.
		 * @param e The ActionEvent object
		 */
		public void actionPerformed(ActionEvent e)
		{
			// Pause while we're doing this
			play.pause();
			// Get file name
			String filename = (String)JOptionPane.showInputDialog(null, "Give a name for the file", "Saving", JOptionPane.PLAIN_MESSAGE);
			if (filename == null)
			{
				play.unpause();
				return;
			}
			// Write the data to the file using serializable
			try
			{
				FileOutputStream saveFile = new FileOutputStream(filename);
				ObjectOutputStream save = new ObjectOutputStream(saveFile);
				save.writeObject(play);
				save.close();
			}
			// If there is some io exception
			catch (IOException io)
			{
				JOptionPane.showMessageDialog(null, "Sorry, there was an error in writing");
				io.printStackTrace();
			}
			// Unpause
			play.unpause();
		}
	}
	
	/**
	 * Private class for firing events when the load button is pressed
	 */
	private class LoadButtonListener implements ActionListener
	{
		/**
		 * Method invoked when the button is pressed. Loads the game in a file
		 * specified by the user.
		 * @param e The ActionEvent object
		 */
		public void actionPerformed(ActionEvent e)
		{
			// Pause while we're doing this
			play.pause();
			// Get file name
			String filename = (String)JOptionPane.showInputDialog(null, "Give the file name", "Loading", JOptionPane.PLAIN_MESSAGE);
			if (filename == null)
			{
				play.unpause();
				return;
			}
			// Read the data to the file using serializable
			try
			{
    	 		FileInputStream saveFile = new FileInputStream(filename);
				ObjectInputStream save = new ObjectInputStream(saveFile);
				PlayingPanel loadedPanel = (PlayingPanel) save.readObject();
   			save.close();
				play.load(loadedPanel);
			}
			// If the file is not in the current directory
			catch(FileNotFoundException fnf)
			{
   			JOptionPane.showMessageDialog(null, "Sorry, could not find that file");
  			}
			// If there is some exception
			catch(Exception ex)
			{
				JOptionPane.showMessageDialog(null, "Sorry, error in reading");
				ex.printStackTrace();
			}
			// Unpause
			play.unpause();
		}
	}
		
	/**
	 * Private class for firing events when the pause button is pressed
	 */	
	private class PauseButtonListener implements ActionListener
	{
		/**
		 * Method invoked when the button is pressed. Pauses the game.
		 * @param e The ActionEvent object
		 */
		public void actionPerformed(ActionEvent e)
		{
			if (play.getPaused())
			{
				// Unpauses the panel
				play.unpause();
			}
			else
			{
				// Pauses the panel
				play.pause();
			}
		}
	}
}